; SketchyLISP Extension Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; factorial

; ---conformance---
; SketchyLISP Extension

; ---purpose---
; Compute the factorial of a number.
; This procedure uses the "recursive product" algorithm.
; The argument must be a positive integer.
; .P
; The algorithm (in Java) was found under
; "factorial functions" at
; .U http://www.luschny.de/math/

; ---args---
; N - number

; ---keywords---
; FACTORIAL function, factorial

; ---see-also---
; permute, product

; ---example---
; (factorial 5) => 120

; (require "lib/digits.scm")
; (require "lib/zerop.scm") ; zero?
; (require "lib/negativep.scm") ; negative?

; ---model---
; (define (factorial n) 
;   (cond ((zero? n) 1)
;     (#t (* n (factorial (- n 1))))))

; ---code---
(define (factorial n)
  (letrec
    ((r*
       (lambda (n k)
         (cond ((< k 2) n)
           (else (let ((l (quotient k 2)))
                   (* (r* n l)
                      (r* (+ n l) (- k l)))))))))
    (cond ((negative? n)
        (bottom (list 'factorial n)))
      (else (r* 1 n)))))
