; SketchyLISP Extension Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; iota

; ---conformance---
; SketchyLISP Extension

; ---purpose---
; Generate sequences of numbers.

; ---args---
; l - lower limit of sequence
; h - upper limit of sequence

; ---keywords---
; IOTA function, sequence of numbers, arithmetics, natural

; ---see-also---
; sum, product

; ---example---
; (iota 1 10) => (1 2 3 4 5 6 7 8 9 10)

; (require "lib/digits.scm")
; (require "lib/list.scm")
; (require "lib/append.scm")
; (require "lib/natural.scm")
; (require "lib/equal.scm") ; =
; (require "lib/plus.scm") ; +
; (require "lib/minus.scm") ; -

; ---model---
; (define (iota l h)
;   (cond ((= l h) (list l))
;     (else (cons l (iota (+ l 1) h)))))

; ---code---
(define (iota l h)
  (letrec
    ((j
       (lambda (x r)
         (cond ((= x l) (cons l r))
           (else (j (- x 1) (cons x r)))))))
    (cond ((> l h)
        (bottom '(iota: bad range)))
      (else (j h '())))))
