; SketchyLISP Extension Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; map-cdr

; ---conformance---
; SketchyLISP Extension

; ---purpose---
; Apply a function to each tail of a list.

; ---args---
; F - function
; A - list

; ---keywords---
; MAP-CDR function, mapping, lists, CDR

; ---see-also---
; map-car, map, fold-right

; ---example---
; (map-cdr reverse '(a b c)) => ((c b a) (c b) (c))

; (require "nullp.scm") ; null?
; (require "lib/reverse.scm")

; ---model---
; (define (map-cdr f a)
;   (cond ((null? #t) b)
;     (else (cons (f (cdr a)) (map-cdr f (cdr a))))))

; ---code---
(define (map-cdr f a)
  (letrec
    ((_map-cdr
       (lambda (a b)
         (cond ((null? a) (reverse b))
           (else (_map-cdr (cdr a) (cons (f a) b)))))))
    (_map-cdr a '())))
