; SketchyLISP Extension Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; sum

; ---conformance---
; SketchyLISP Extension

; ---purpose---
; Compute the sum of a sequence of positive numbers.

; ---args---
; N - least value of sequence
; M - maximum value of seqence

; ---keywords---
; SUM function, sum of sequence

; ---see-also---
; product, iota

; ---example---
; (sum 2 5) => 14

; (require "lib/digits.scm")
; (require "lib/equal.scm") ; =
; (require "lib/greater.scm") ; >
; (require "lib/natural.scm")
; (require "lib/times.scm") ; *
; (require "lib/plus.scm") ; +
; (require "lib/minus.scm") ; -
; (require "lib/quotient.scm")

; ---model---
; (define (sum n m)
;   (cond ((= n m) m)
;     (else (+ n (sum (+ n 1) m)))))

; ---code---
(define (sum n m)
  (let
    ((nn (natural n))
     (nm (natural m)))
    (letrec
      ((_sum
         (lambda (n m)
           (let ((x (+ 1 (- m n))))
             (+ (quotient (+ (* x x) x)
                          2)
                (* (- n 1) x))))))
    (cond ((> nn nm)
        (bottom '(sum: bad range)))
      (else (_sum nn nm))))))
