; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; assq

; ---conformance---
; R5RS

; ---purpose---
; Retrieve an association from an association list.
; An association list is a list of pairs where
; the
; .E car
; part of each pair holds a key and the
; .E cdr
; part of the pair holds the value
; associated with that key:
; .B
; .C ((key1 . value1) ... (keyN . valueN))
; .B
; .C Assq
; returns the first association whose key is identical
; to a given symbol.

; ---args---
; X - key of value to be found
; A - association list

; ---keywords---
; ASSQ function, association lists, alists, associate

; ---see-also---
; assoc, memq

; ---example---
; (assq 'c '((a . i) (b . ii) (c . iii) (d . iv))) => (c . iii)

(define assq #t)

; (require "nullp.scm") ; null?
(require "caar.scm") ; cdar

; ---code---
(define (assq x a)
  (cond ((null? a) #f)
    ((eq? (caar a) x) (car a))
    (else (assq x (cdr a)))))
