; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; expt

; ---conformance---
; R5RS

; ---purpose---
; Raise a number to a power.
; Return
; .V x
; raised to the power of
; .V y.
; .V X
; may be a natural or integer number.
; .V Y
; must be natural.

; ---args---
; X - number (base)
; Y - number (exponent)

; ---keywords---
; EXPT function, base, exponent, power

; ---see-also---
; digits, sqrt, gcd

; ---example---
; (expt 2 5) => 32

(define expt #t)

(require "digits.scm")
(require "negativep.scm") ; negative?
(require "abs.scm")
(require "zerop.scm") ; zero?
(require "evenp.scm") ; even?
(require "times.scm") ; *
(require "divide.scm")
(require "nquotient.scm")
(require "integer.scm")

; ---model---
; (define (expt x y)
;   (letrec
;     ((exp
;        (lambda (x y r)
;          (cond ((zero? y) r)
;            (else (exp x (- y 1) (* x r)))))))
;     (exp (integer x) (natural y) 1)))

; ---code---
(define (expt x y)
  (letrec
    ((square
       (lambda (x)
         (* x x)))
     (_expt
       (lambda (x y)
         (cond ((zero? y) 1)
           ((even? y)
             (square (_expt x (nquotient y 2))))
           (else (* x (square (_expt x (nquotient y 2)))))))))
    (_expt (integer x) (natural y))))
