; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; >

; ---conformance---
; R5RS

; ---purpose---
; Check whether a sequence of numbers is in strict descending
; order. Return
; .C #t,
; if
; .V a>b>...
; and otherwise
; .C #f.

; ---args---
; A - number
; B... - numbers

; ---keywords---
; > function, greater than, ordering
; relation, comparison, predicate

; ---see-also---
; digits, <, <=, >=, n>

; ---example---
; (> 7 5 3) => #t

(define greater #t)

; (require "nullp.scm") ; null?
(require "less.scm") ; <
(require "fold-left.scm")
(require "neqp.scm") ; neq?

; ---code---
(define (> a . b)
  (letrec
    ((gt
       (lambda (a b)
         (cond ((eq? a #t) #t)
           ((< b a) b)
           (else #t)))))
    (cond ((null? b)
        (bottom '(too few arguments to >)))
      (else (neq? (fold-left gt a b) #t)))))
