; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; <=

; ---conformance---
; R5RS

; ---purpose---
; Check whether a seqeunce of numbers is in strict
; non-descending order.
; Return
; .C #t,
; if
; .V a<=b<=...
; and otherwise
; .C #f.

; ---args---
; A - number
; B... - numbers

; ---keywords---
; <= function, not greater than, less than or equal to,
; ordering, relation, comparison, predicate

; ---see-also---
; digits, <, >, >=, n<=

; ---example---
; (<= 9 9 10) => #t

(define lteq #t)

; (require "nullp.scm") ; null?
(require "greater.scm") ; >
(require "fold-left.scm")
(require "neqp.scm") ; neq?

; ---code---
(define (<= a . b)
  (letrec
    ((lteq
       (lambda (a b)
         (cond ((eq? a #t) #t)
           ((> a b) #t)
           (else b)))))
    (cond ((null? b)
        (bottom '(too few arguments to <=)))
      (else (neq? (fold-left lteq a b) #t)))))
