; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; n-

; ---conformance---
; SketchyLISP Core

; ---purpose---
; Subtract two natural numbers.

; ---args---
; A - natural number
; B - natural number to be subtracted

; ---keywords---
; N- function, difference, subtract, subtraction
; arithmetics, natural

; ---see-also---
; digits, n+, nquotient, nremainder, n*, -

; ---example---
; (n- 7 5) => 2

(define nminus #t)

; (require "nullp.scm") ; null?
; (require "digits.scm") ; 0d 1d 9d
; (require "append.scm")
; (require "reverse.scm")
; (require "normalize.scm")
; (require "pred.scm")

; ---model---
; (define (n- a b)
;   (letrec
;     ; Compute difference of two digits with borrow
;     ; X Y - digits
;     ; CI CO - borrow (carry) in, out
;     ; RESULT - (difference . carry out)
;     ((d-
;        (lambda (x y ci co)
;          (cond ((eq? y 0d)
;              (cond ((eq? ci 1d)
;                  ; subtract carry in
;                  (d- x 1d 0d co))
;                (else (cons x co))))
;            ; handle overflow
;            ((null? (pred x))
;              (d- 9d (pred y) ci 1d))
;            (else (d- (pred x) (pred y) ci co)))))
;      ; Subtract two lists of decimal digits forming reverse
;      ; decimal numbers. The result is in normal order.
;      ; A B - reverse lists of digits
;      ; C - carry flag
;      ; R - result (normal order)
;      (diff
;        (lambda (a b c r)
;          (cond ((null? a)
;              (cond
;                ; A=() ; B=()
;                ((null? b)
;                  (cond
;                    ; carry not set? OK
;                    ((eq? c 0d) r)
;                    ; A<B is undefined
;                    (else (bottom 'difference-negative))))
;                (else (bottom 'difference-negative))))
;            ; A\=() ; B=() ; see above
;            ((null? b)
;              (cond
;                ; carry not set?
;                ((eq? c 0d)
;                  ; prepend rest of A
;                  (append (reverse a) r))
;                ; else subtract borrow
;                (else (diff a (integer->list 1) 0d r))))
;            ; advance to next pair of digits
;            (else (diff (cdr a) (cdr b)
;                    ; compute borrow
;                    (cdr (d- (car a) (car b) c 0d))
;                    ; prepend result digit
;                    (cons (car (d- (car a) (car b) c 0d))
;                      r)))))))
;     (normalize
;       (list->integer
;         (diff (reverse (integer->list a))
;               (reverse (integer->list b))
;               0d '())
;         #t))))

; ---code---
; This function is a primitive function.
