; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; normalize

; ---conformance---
; SketchyLISP Core

; ---purpose---
; Normalize a number by removing leading zeroes and plus signs.

; ---args---
; X - number

; ---keywords---
; NORMALIZE function, normalize, normal form, canonical number
; GCD, reducing, numbers, digits

; ---see-also---
; digits, ndivide

; ---example---
; (normalize +000511) => 511

(define normalize #t)

; (require "nullp.scm") ; null?
(require "digits.scm") ; 0d
(require "caar.scm") ; cdar
(require "caaar.scm") ; caadr cdadr
(require "gcd.scm")
(require "equal.scm") ; =
(require "equalp.scm") ; equal?
(require "quotient.scm")
(require "minus.scm") ; -

; ---code---
(define (normalize x)
  (letrec
    ; remove leading zeroes from natural number
    ((skip0
       (lambda (x)
         (cond ((null? (cdr x)) x)
           ((eq? (car x) 0d)
             (skip0 (cdr x)))
           (else x))))
     (norm
       (lambda (x)
         (cond ((eq? (car x) '+)
             (skip0 (cdr x)))
           ((eq? (car x) '-)
             (let ((d (skip0 (cdr x))))
               (cond ((equal? d '(0d)) d)
                 (else (cons '- d)))))
           (else (skip0 x))))))
    (list->integer
      (norm (integer->list x)) #t)))
