; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; string-ci=?

; ---conformance---
; R5RS

; ---purpose---
; Case-insensitively test whether all strings in a sequence
; are equal.

; ---args---
; X - string
; Y... - strings

; ---keywords---
; STRING-CI=? function, strings, comparison, equivalence,
; lexical equivalence, case-insensitive

; ---see-also---
; string=?, string-ci<?, string-ci>?, string-ci<=?, string-ci>=?, char-ci=?
; equal?

; ---example---
; (string-ci=? "xyz" "XYZ" "xYz") => #t

(define s-cieqp #t)

; (require "c-cieqp.scm") ; char-ci=?
; (load "nullp.scm") ; null?
(require "pred-iter.scm") ; predicate-iterator

; ---code---
(define string-ci=?
  (letrec
    ((ci-eq?
       (lambda (x y)
         (cond ((null? x) (null? y))
           ((null? y) #f)
           (else (and (char-ci=? (car x) (car y))
                      (ci-eq? (cdr x) (cdr y))))))))
    (predicate-iterator
      (lambda (x y)
        (ci-eq? (string->list x)
                (string->list y))))))
