; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; string-ci>=?

; ---conformance---
; R5RS

; ---purpose---
; Case-insensitively test whether a sequence of strings
; is in lexically non-ascending order.

; ---args---
; X - string
; Y... - strings

; ---keywords---
; STRING-CI>=? function, strings, comparison, lexical order,
; non-ascending, case-insensitive

; ---see-also---
; string>=?, string-ci<?, string-ci=?, string-ci>?, string-ci<=?, char-ci>=?
; equal?

; ---example---
; (string-ci>=? "XYZ" "xyz" "xXx") => #t

(define s-cigep #t)

; (require "c-ciltp.scm") ; char-ci<?
; (require "c-cigtp.scm") ; char-ci>?
; (load "nullp.scm") ; null?
(require "pred-iter.scm") ; predicate-iterator

; ---code---
(define string-ci>=?
  (letrec
    ((ci-ge?
       (lambda (x y)
         (cond ((null? x) (null? y))
           ((null? y) #t)
           ((char-ci<? (car x) (car y)) #f)
           ((char-ci>? (car x) (car y)) #t)
           (else (ci-ge? (cdr x) (cdr y)))))))
  (predicate-iterator
    (lambda (x y)
      (ci-ge? (string->list x)
              (string->list y))))))
