; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; string>=?

; ---conformance---
; R5RS

; ---purpose---
; Test whether a sequence of strings is in
; lexically non-ascending order.

; ---args---
; X - string
; Y... - strings

; ---keywords---
; STRING>=? function, strings, comparison, lexical order,
; non-ascending

; ---see-also---
; string-ci>=?, string<?, string=?, string>?, string<=?, char>=?, equal?

; ---example---
; (string>=? "xyz" "xyz" "xxx") => #t

(define sgep #t)

; (require "cltp.scm") ; char<?
; (require "cgtp.scm") ; char>?
; (load "nullp.scm") ; null?
(require "pred-iter.scm") ; predicate-iterator

; ---code---
(define string>=?
  (letrec
    ((ge?
       (lambda (x y)
         (cond ((null? x) (null? y))
           ((null? y) #t)
           ((char<? (car x) (car y)) #f)
           ((char>? (car x) (car y)) #t)
           (else (ge? (cdr x) (cdr y)))))))
  (predicate-iterator
    (lambda (x y)
      (ge? (string->list x)
           (string->list y))))))
