; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; string>?

; ---conformance---
; R5RS

; ---purpose---
; Test whether a sequence of strings is in
; lexically descending order.

; ---args---
; X - string
; Y... - strings

; ---keywords---
; STRING>? function, strings, comparison, lexical order,
; descending

; ---see-also---
; string-ci>?, string<?, string=?, string<=?, string>=?, char>?, equal?

; ---example---
; (string>? "xyz" "abc" "aaa") => #t

(define sgtp #t)

; (require "cltp.scm") ; char<?
; (require "cgtp.scm") ; char>?
; (load "nullp.scm") ; null?
(require "pred-iter.scm") ; predicate-iterator

; ---code---
(define string>?
  (letrec
    ((gt?
       (lambda (x y)
         (cond ((null? x) #f)
           ((null? y) #t)
           ((char<? (car x) (car y)) #f)
           ((char>? (car x) (car y)) #t)
           (else (gt? (cdr x) (cdr y)))))))
  (predicate-iterator
    (lambda (x y)
      (gt? (string->list x)
           (string->list y))))))
