; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; string<?

; ---conformance---
; R5RS

; ---purpose---
; Test whether a sequence of strings is in
; lexically ascending order.

; ---args---
; X - string
; Y... - strings

; ---keywords---
; STRING<? function, strings, comparison, lexical order,
; ascending

; ---see-also---
; string-ci<?, string=?, string>?, string<=?, string>=?, char<?, equal?

; ---example---
; (string<? "abc" "xyz" "zzz") => #t

(define sltp #t)

; (require "cgtp.scm") ; char>?
; (require "cltp.scm") ; char<?
; (load "nullp.scm") ; null?
(load "not.scm")
(require "pred-iter.scm") ; predicate-iterator

; ---code---
(define string<?
  (letrec
    ((lt?
       (lambda (x y)
         (cond ((null? x) (not (null? y)))
           ((null? y) #f)
           ((char<? (car x) (car y)) #t)
           ((char>? (car x) (car y)) #f)
           (else (lt? (cdr x) (cdr y)))))))
  (predicate-iterator
    (lambda (x y)
      (lt? (string->list x)
           (string->list y))))))
