; SketchyLISP Example Program
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; insert

; ---language---
; R5RS Scheme

; ---purpose---
; Insert a member into an ordered list.
; The predicate
; .V p
; must be asymmetrical and evaluate to
; .C #t
; for each two subsequent members of
; .V a.

; ---args---
; X - member to insert
; A - ordered list
; P - predicate defining the order of A

; ---keywords---
; INSERT function, sorting, insertion sort
; ordering, lists, ordered lists

; ---example---
; (insert < 4 '(3 5)) => (3 4 5)

; (require "lib/append.scm")
; (require "lib/reverse.scm")

; ---model---
; (define (insert p x a)
;   (cond ((null? a) (cons x '()))
;     ((p x (car a)) (cons x a))
;     (else (cons (car a) (insert p x (cdr a))))))

; ---code---
(define (insert p x a)
  (letrec
    ((_insert
       (lambda (a r)
         (cond ((null? a)
             (reverse (cons x r)))
           ((p x (car a))
             (append (reverse (cons x r)) a))
           (else (_insert (cdr a)
                   (cons (car a) r)))))))
    (_insert a '())))
