; SketchyLISP Example Program
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; unlet

; ---language---
; R5RS Scheme

; ---purpose---
; Convert a let expression into an equivalent
; lambda expression.

; ---args---
; X - expression to convert

; ---keywords---
; LET, LAMBDA, scoping, example

; ---example---
; (unlet '(let ((a 5) (b 7)) (+ a b))) => ((lambda (a b) (+ a b)) 5 7)

; (require "lib/list.scm")
; (require "lib/append.scm")
; (require "lib/map.scm")
; (require "lib/caar.scm") ; cadr
; (require "lib/caaar.scm") ; caddr

; ---code---
(define (unlet x)
  (letrec
    ((make-lambda
       (lambda (let-expr)
         (let ((env (cadr let-expr)))
           (let ((formals (map car env))
                 (actuals (map (lambda (x) (unlet (cadr x)))
                               env))
                 (body (unlet (caddr let-expr))))
             (append (list (list 'lambda formals body))
                     actuals))))))
    (cond ((not (pair? x)) x)
      ((eq? (car x) 'quote) x)
      ((eq? (car x) 'let) (make-lambda x))
      ((eq? (car x) 'lambda) ; handle dotted formals
        (list 'lambda (cadr x) (unlet (caddr x))))
      (else (map unlet x)))))
